/* helpfile.h - Help file loading and display
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: helpfile.h,v 1.13.2.5 2001/08/14 15:31:38 entrope Exp $
 */

#if !defined(HELPFILE_H)
#define HELPFILE_H

struct helpfile;

struct userNode;
struct handle_info;

struct string_list;

#define MIN_LINE_SIZE		40
#define MAX_LINE_SIZE		450

#define DEFAULT_LINE_SIZE	MAX_LINE_SIZE
#define COLUMN_LINE_SIZE	40

typedef char* (*expand_func_t)(char *variable);

#ifdef __GNUC__
#define PRINTF_LIKE(M,N) __attribute__((format (printf, M, N)))
#else
#define PRINTF_LIKE(M,N)
#endif

int send_message(struct userNode *dest, struct userNode *src, const char *message, ...) PRINTF_LIKE(3,4);
int send_target_message(int use_privmsg, const char *dest, struct userNode *src, const char *format, ...) PRINTF_LIKE(4,5);
int send_help(struct userNode *dest, struct userNode *src, struct helpfile *hf, const char *topic);
void column_send(struct userNode *service, struct userNode *user, struct string_list *list);

#define send_channel_message(CHANNEL, ARGS...) send_target_message(1, (CHANNEL)->name, ARGS)
#define send_server_message(SERVER, ARGS...) send_target_message(1, (SERVER)->name, ARGS)

#define send_channel_notice(CHANNEL, ARGS...) send_target_message(0, (CHANNEL)->name, ARGS)
#define send_server_notice(SERVER, ARGS...) send_target_message(0, (SERVER)->name, ARGS)

struct helpfile *open_helpfile(const char *fname, expand_func_t expand);
void close_helpfile(struct helpfile *hf);

#endif
