/* policer.c - Leaky bucket
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: policer.c,v 1.11.2.1 2001/03/29 00:20:52 zoot Exp $
 */

#include "config.h"
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "common.h"
#include "policer.h"

/* This policer uses the "leaky bucket" (GCRA) algorithm. */ 

struct policer_params {
    double bucket_size;
    double drain_rate;
};

struct policer_params *
policer_params_new(void)
{
    struct policer_params *params = malloc(sizeof(struct policer_params));
    params->bucket_size = 0.0;
    params->drain_rate = 0.0;
    return params;
}

int
policer_params_set(struct policer_params *params, const char *param, const char *value)
{
    if (!strcasecmp(param, "size")) {
	params->bucket_size = strtod(value, NULL);
    } else if (!strcasecmp(param, "drain-rate")) {
	params->drain_rate = strtod(value, NULL);
    } else {
	return 0;
    }
    return 1;
}

void
policer_params_delete(struct policer_params *params)
{
    free(params);
}

struct policer *
policer_new(struct policer_params *params, policer_time_t now)
{
    struct policer *pol = malloc(sizeof(struct policer));
    pol->params = params;
    pol->level = 0.0;
    pol->last_req = now;
    return pol;
}

int
policer_conforms(struct policer *pol, policer_time_t reqtime, double weight)
{
    pol->level -= pol->params->drain_rate * (reqtime - pol->last_req);
    if (pol->level < 0.0) pol->level = 0.0;
    pol->level += weight;
    pol->last_req = reqtime;
    return pol->level < pol->params->bucket_size;
}

void
policer_delete(struct policer *pol)
{
    free(pol);
}
